@@ -28,10 +28,6 @@
 #include "amdgpu.h"
 #include "amdgpu_ucode.h"
 
-static const struct kicker_device kicker_device_list[] = {
-	{0x744B, 0x00},
-};
-
 static void amdgpu_ucode_print_common_hdr(const struct common_firmware_header *hdr)
 {
 	DRM_DEBUG("size_bytes: %u\n", le32_to_cpu(hdr->size_bytes));
@@ -1063,19 +1059,6 @@ int amdgpu_ucode_init_bo(struct amdgpu_device *adev)
 	return 0;
 }
 
-bool amdgpu_is_kicker_fw(struct amdgpu_device *adev)
-{
-	int i;
-
-	for (i = 0; i < ARRAY_SIZE(kicker_device_list); i++) {
-		if (adev->pdev->device == kicker_device_list[i].device &&
-			adev->pdev->revision == kicker_device_list[i].revision)
-		return true;
-	}
-
-	return false;
-}
-
 void amdgpu_ucode_ip_version_decode(struct amdgpu_device *adev, int block_type, char *ucode_prefix, int len)
 {
 	int maj, min, rev;
@@ -1127,13 +1110,9 @@ int amdgpu_ucode_request(struct amdgpu_device *adev, const struct firmware **fw,
 
 	if (err)
 		return -ENODEV;
-
 	err = amdgpu_ucode_validate(*fw);
-	if (err) {
+	if (err)
 		dev_dbg(adev->dev, "\"%s\" failed to validate\n", fw_name);
-		release_firmware(*fw);
-		*fw = NULL;
-	}
 
 	return err;
 }
